package ee.yoursit.core;

import dev.inventex.discord.DiscordClient;
import dev.inventex.discord.DiscordClientBuilder;
import ee.yoursit.core.config.Config;
import ee.yoursit.core.config.Messages;
import ee.yoursit.core.data.DataManager;
import ee.yoursit.core.data.PlayerData;
import ee.yoursit.core.database.Database;
import ee.yoursit.core.database.service.*;
import ee.yoursit.core.module.*;
import lombok.Getter;
import net.dv8tion.jda.api.requests.GatewayIntent;
import org.bukkit.Bukkit;
import org.bukkit.Location;
import org.bukkit.plugin.java.JavaPlugin;

import javax.security.auth.login.LoginException;

@Getter
public enum YsmpCore {
    INSTANCE;

    private JavaPlugin plugin;

    private Config config, messages;
    private DataManager dataManager;
    private Database database;

    private Location spawnLoc, verifyLoc;

    private UserService userService;
    private GroupService groupService;
    private InviteService inviteService;
    private BanService banService;
    private VerificationService verificationService;

    private CommandRegistry bukkitCommands, discordCommands;

    private DiscordClient client;

    public void enable(JavaPlugin plugin) {
        this.plugin = plugin;

        loadConfig();
        loadDatabase();

        createDiscordClient();

        loadCommands();

        Bukkit
            .getPluginManager()
            .registerEvents(new BukkitListener(), plugin);
    }

    private void loadConfig() {
        plugin.getDataFolder().mkdir();

        config = new Config("config");
        messages = new Config("messages");

        config.load();
        messages.load();

        Messages.init(messages);

        Bukkit
            .getScheduler()
            .runTask(plugin, () -> {
                spawnLoc = config.getLocation("spawn");
                verifyLoc = config.getLocation("verify");
            });
    }

    private void loadDatabase() {
        dataManager = new DataManager();
        database = new Database("127.0.0.1", 27017);
        database.connect();

        userService = new UserService();
        groupService = new GroupService();
        groupService.loadGroups();
        inviteService = new InviteService();
        banService = new BanService();
        verificationService = new VerificationService();
    }

    private void loadCommands() {
        bukkitCommands = new BukkitCommandRegistry();
        bukkitCommands.load();

        discordCommands = new DiscordCommandRegistry();
    }

    private void createDiscordClient() {
        try {
            String token = config.getString("bot-token");
            client = new DiscordClientBuilder(token)
                .useMembers()
                .useRoles()
                .enableIntents(GatewayIntent.MESSAGE_CONTENT)
                .build();
            client.registerEvents(new DiscordListener());
        } catch (LoginException e) {
            System.err.println("Unable to create discord client");
            e.printStackTrace();
        }
    }

    public void disable() {
        database.disconnect();

        bukkitCommands.unload();
        discordCommands.unload();

        client.getJda().shutdown();
    }
}
