package ee.yoursit.core.command.bukkit.impl.admin;

import dev.inventex.octa.concurrent.future.Future;
import dev.inventex.octa.data.convertible.Convertible;
import ee.yoursit.core.YsmpCore;
import ee.yoursit.core.command.bukkit.api.Command;
import ee.yoursit.core.command.bukkit.api.CommandContext;
import ee.yoursit.core.command.bukkit.api.CommandInfo;
import ee.yoursit.core.command.bukkit.api.Completion;
import ee.yoursit.core.command.bukkit.impl.admin.group.CreateArgument;
import ee.yoursit.core.command.bukkit.impl.admin.group.DeleteArgument;
import ee.yoursit.core.command.bukkit.impl.admin.group.ListArgument;
import ee.yoursit.core.config.Messages;
import ee.yoursit.core.database.entity.impl.Group;
import ee.yoursit.core.database.entity.impl.User;
import ee.yoursit.core.util.Message;
import org.bukkit.Bukkit;
import org.bukkit.entity.Player;
import org.jetbrains.annotations.NotNull;

import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

@CommandInfo(name = "group")
public class GroupCommand extends Command {
    @Override
    public void create() {
        addArgument(new CreateArgument());
        addArgument(new DeleteArgument());
        addArgument(new ListArgument());
    }

    // TODO reword command system, so I don't need to do these manually >:(

    @Override
    public void onCommand(@NotNull CommandContext context) throws Exception {
        if (!(context.getSender().hasPermission("ysmp.admin"))) {
            context.reply(Messages.NO_PERMISSION);
            return;
        }

        Convertible<String, String> groupArg = context.getString(0);
        if (groupArg.isEmpty()) {
            context.reply(Messages.GROUP_USAGE);
            return;
        }

        String name = groupArg.get();
        YsmpCore.INSTANCE
            .getGroupService()
            .getGroup(name)
            .tryThen(group -> handleGroup(context, group))
            .except(err ->
                context.reply(Messages.GROUP_NOT_EXISTS
                    .replace("{group}", name)));
    }

    private void handleGroup(CommandContext context, Group group) throws Exception {
        Convertible<String, String> commandArg = context.getString(1);
        if (commandArg.isEmpty()) {
            context.reply(Messages.GROUP_USAGE);
            return;
        }

        switch (commandArg.get()) {
            case "meta" -> handleGroupMeta(context, group);
            case "permission" -> handleGroupPermission(context, group);
            case "grant" -> handleGroupGrant(context, group);
            default -> context.reply(Messages.GROUP_USAGE);
        }
    }

    private void handleGroupMeta(CommandContext context, Group group) throws Exception {
        Convertible<String, String> metaTypeArg = context.getString(2);
        if (metaTypeArg.isEmpty()) {
            context.reply(Messages.GROUP_META_PREFIX_USAGE);
            return;
        }

        switch (metaTypeArg.get()) {
            case "prefix" -> handleMetaPrefix(context, group);
            case "suffix" -> handleMetaSuffix(context, group);
            case "tabprefix" -> handleMetaTabPrefix(context, group);
            case "tabsuffix" -> handleMetaTabSuffix(context, group);
            case "info" -> handleMetaInfo(context, group);
            default -> context.reply(Messages.GROUP_META_USAGE);
        }
    }

    private void handleMetaPrefix(CommandContext context, Group group) throws Exception {
        if (context.getString(3).isEmpty()) {
            context.reply(Messages.GROUP_META_PREFIX_USAGE);
            return;
        }

        String prefix = Arrays
            .stream(context.getData())
            .skip(3)
            .collect(Collectors.joining(" "));

        group.getPrefix().set(prefix);
        prefix = Message.translate(prefix);

        context.reply(Messages.GROUP_META_PREFIX_SET
            .replace("{group}", group.getName().get())
            .replace("{prefix}", prefix));
    }

    private void handleMetaSuffix(CommandContext context, Group group) throws Exception {
        if (context.getString(3).isEmpty()) {
            context.reply(Messages.GROUP_META_SUFFIX_USAGE);
            return;
        }

        String suffix = Arrays
            .stream(context.getData())
            .skip(3)
            .collect(Collectors.joining(" "));

        group.getSuffix().set(suffix);
        suffix = Message.translate(suffix);

        context.reply(Messages.GROUP_META_SUFFIX_SET
            .replace("{group}", group.getName().get())
            .replace("{suffix}", suffix));
    }

    private void handleMetaTabPrefix(CommandContext context, Group group) throws Exception {
        if (context.getString(3).isEmpty()) {
            context.reply(Messages.GROUP_META_TAB_PREFIX_USAGE);
            return;
        }

        String tabPrefix = Arrays
            .stream(context.getData())
            .skip(3)
            .collect(Collectors.joining(" "));

        group.getTabPrefix().set(tabPrefix);
        tabPrefix = Message.translate(tabPrefix);

        context.reply(Messages.GROUP_META_TAB_PREFIX_SET
            .replace("{group}", group.getName().get())
            .replace("{tabprefix}", tabPrefix));
    }

    private void handleMetaTabSuffix(CommandContext context, Group group) throws Exception {
        if (context.getString(3).isEmpty()) {
            context.reply(Messages.GROUP_META_TAB_SUFFIX_USAGE);
            return;
        }

        String tabSuffix = Arrays
            .stream(context.getData())
            .skip(3)
            .collect(Collectors.joining(" "));

        group.getTabSuffix().set(tabSuffix);
        tabSuffix = Message.translate(tabSuffix);

        context.reply(Messages.GROUP_META_TAB_SUFFIX_SET
            .replace("{group}", group.getName().get())
            .replace("{tabsuffix}", tabSuffix));
    }

    private void handleMetaInfo(CommandContext context, Group group) {
        String name = group.getName().get();
        String prefix = group.getPrefix().get();
        String suffix = group.getSuffix().get();
        String tabPrefix = group.getTabPrefix().get();
        String tabSuffix = group.getTabSuffix().get();

        String message = Messages.GROUP_META_INFO
            .replace("{group}", Message.translate(name))
            .replace("{prefix}", Message.translate(prefix != null ? prefix : "not set"))
            .replace("{suffix}", Message.translate(suffix != null ? suffix : "not set"))
            .replace("{tabprefix}", Message.translate(tabPrefix != null ? tabPrefix : "not set"))
            .replace("{tabsuffix}", Message.translate(tabSuffix != null ? tabSuffix : "not set"));

        context.reply(message);
    }

    private void handleGroupPermission(CommandContext context, Group group) throws Exception {
        Convertible<String, String> actionArg = context.getString(4);
        if (actionArg.isEmpty()) {
            context.reply(Messages.GROUP_PERMISSION_USAGE);
            return;
        }

        switch (actionArg.get()) {
            case "set" -> handleGroupPermissionSet(context, group);
            case "unset" -> handleGroupPermissionUnset(context, group);
            default -> context.reply(Messages.GROUP_PERMISSION_USAGE);
        }
    }

    private void handleGroupPermissionSet(CommandContext context, Group group) throws Exception {
        Convertible<String, String> permissionArg = context.getString(5);
        if (permissionArg.isEmpty()) {
            context.reply(Messages.GROUP_PERMISSION_SET_USAGE);
            return;
        }

        Convertible<String, Boolean> stateArg = context.getBoolean(6);
        if (stateArg.isEmpty()) {
            context.reply(Messages.GROUP_PERMISSION_SET_USAGE);
            return;
        }

        String permission = permissionArg.get();
        Boolean state = stateArg.get();

        group.getPermissions().set(permission, state);

        context.reply(Messages.GROUP_PERMISSION_SET_UPDATED
            .replace("{group}", group.getName().get())
            .replace("{permission}", permission)
            .replace("{state}", String.valueOf(state)));
    }

    private void handleGroupPermissionUnset(CommandContext context, Group group) throws Exception {
        Convertible<String, String> permissionArg = context.getString(5);
        if (permissionArg.isEmpty()) {
            context.reply(Messages.GROUP_PERMISSION_UNSET_USAGE);
            return;
        }

        String permission = permissionArg.get();
        group.getPermissions().unset(permission);

        context.reply(Messages.GROUP_PERMISSION_UNSET_UPDATED
            .replace("{group}", group.getName().get())
            .replace("{permission}", permission));
    }

    private void handleGroupGrant(CommandContext context, Group group) throws Exception {
        Convertible<String, String> playerArg = context.getString(2);
        if (playerArg.isEmpty()) {
            context.reply(Messages.GROUP_GRANT_USAGE);
            return;
        }

        String player = playerArg.get();
        Future<User> future;

        Player onlinePlayer = Bukkit.getPlayerExact(player);
        if (onlinePlayer != null) {
            future = YsmpCore.INSTANCE
                .getDataManager()
                .get(onlinePlayer)
                .getUser();
        } else {
            future = YsmpCore.INSTANCE
                .getUserService()
                .getUser(player);
        }

        future
            .then(user -> {
                user.getGroup().set(group.getName().get());
                context.reply(Messages.GROUP_GRANT_GRANTED
                    .replace("{group}", group.getName().get())
                    .replace("{player}", player));
            })
            .except(err -> context.reply(Messages.NO_SUCH_USER
                .replace("{name}", player)));
    }

    @Override
    public Completion onTabComplete(@NotNull CommandContext context) throws Exception {
        if (!(context.getSender().hasPermission("ysmp.admin")))
            return Completion.EMPTY;

        // this command system is a piece of shit, so I have to do it manually
        if (context.size() == 0) {
            Completion completion = Completion.of("create", "delete");
            List<String> groups = YsmpCore.INSTANCE
                .getGroupService()
                .getGroupCache()
                .values()
                .stream()
                .map(group -> group.getName().get())
                .toList();
            completion.getData().addAll(groups);
            return completion;
        }

        else if (context.size() == 1) {
            String arg = context.getString(0).get();
            if (arg.equals("create") || arg.equals("delete"))
                return Completion.EMPTY;
            return Completion.of("meta", "permission", "grant");
        }

        else if (context.size() == 2) {
            String action = context.getString(1).get();
            if (action.equals(""))
                return Completion.of("meta", "permission", "grant");

            if (action.startsWith("m"))
                return Completion.of("meta");
            else if (action.startsWith("p"))
                return Completion.of("permission");
            else if (action.startsWith("g"))
                return Completion.of("grant");
        }

        else if (context.size() == 3) {
            String action = context.getString(1).get();

            return switch (action) {
                case "meta" -> Completion.of("info", "prefix", "suffix", "tabprefix", "tabsuffix");
                case "permission" -> Completion.of("set", "unset");
                case "grant" -> Completion.PLAYERS;
                default -> Completion.EMPTY;
            };
        }

        return Completion.EMPTY;
    }
}
