package ee.yoursit.core.command.discord;

import dev.inventex.discord.command.slash.CommandContext;
import dev.inventex.discord.command.slash.SlashCommand;
import dev.inventex.discord.command.slash.SlashCommandInfo;
import ee.yoursit.core.YsmpCore;
import ee.yoursit.core.config.Messages;
import ee.yoursit.core.database.entity.api.EntitySerializer;
import ee.yoursit.core.database.entity.impl.Invite;
import ee.yoursit.core.database.entity.impl.Verification;
import ee.yoursit.core.util.Constants;
import ee.yoursit.core.util.InviteGenerator;
import lombok.SneakyThrows;
import net.dv8tion.jda.api.EmbedBuilder;
import net.dv8tion.jda.api.entities.Guild;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.MessageEmbed;
import net.dv8tion.jda.api.entities.Role;
import net.dv8tion.jda.api.interactions.commands.OptionType;

import java.awt.*;

@SlashCommandInfo(name = "smp", description = "Invite yourself to the YourSitee SMP server.")
public class SmpCommand extends SlashCommand {
    private final InviteGenerator generator = new InviteGenerator(5, 4);

    @Override
    protected void initialize() {
        addOption(OptionType.STRING,
            "username",
            "The in-game name of your Minecraft account",
            true, false
        );
    }

    @Override
    @SneakyThrows
    public void execute(CommandContext context) {
        Member member = context
            .getInteraction()
            .getMember();
        assert member != null;

        Guild guild = YsmpCore.INSTANCE
            .getClient()
            .getJda()
            .getGuildById(Constants.YSMP_SERVER);
        assert guild != null;

        Member smpMember = guild.getMember(member.getUser());
        if (smpMember != null
            && smpMember
                .getRoles()
                .stream()
                .anyMatch(role -> role.getIdLong() == Constants.PLAYER)
        ) {
            MessageEmbed embed = new EmbedBuilder()
                .setTitle("Error")
                .setDescription(Messages.ALREADY_USER)
                .setColor(Color.RED)
                .build();

            context
                .getInteraction()
                .deferReply(true)
                .addEmbeds(embed)
                .queue();

            return;
        }

        Verification existingVerification = YsmpCore.INSTANCE
            .getVerificationService()
            .getVerification(member.getIdLong())
            .getOrDefault(null);

        if (existingVerification != null) {
            MessageEmbed embed = new EmbedBuilder()
                .setTitle("Error")
                .setDescription(Messages.INVITE_ALREADY_VERIFYING_SELF)
                .setColor(Color.RED)
                .build();

            context
                .getInteraction()
                .deferReply(true)
                .addEmbeds(embed)
                .queue();

            return;
        }

        String code = generator.generate();

        Invite invite = EntitySerializer.init(Invite.class);
        invite.getCode().init(code);
        invite.getCreator().init("Server");
        invite.getUsed().init(false);

        YsmpCore.INSTANCE
            .getInviteService()
            .createInvite(invite)
            .then(e -> invite(context, member, code))
            .except(System.err::println);
    }

    @SneakyThrows
    private void invite(CommandContext context, Member member, String code) {
        String username = context.getString("username");

        Verification verification = EntitySerializer.init(Verification.class);
        verification.getDiscordId().init(member.getIdLong());
        verification.getUsername().init(username);
        verification.getCode().init(code);
        verification.getCreation().init(System.currentTimeMillis());

        YsmpCore.INSTANCE
            .getVerificationService()
            .addVerification(verification)
            .get();

        MessageEmbed embed = new EmbedBuilder()
            .setTitle("You have been invited")
            .setDescription(Messages.INVITE_VERIFICATION
                .replace("{code}", code))
            .setColor(Constants.TRANSPARENT)
            .build();

        String discordLink = YsmpCore.INSTANCE
            .getConfig()
            .getString("ysmp-invite");

        context
            .getInteraction()
            .deferReply(true)
            .addEmbeds(embed)
            .addContent(discordLink)
            .queue();
    }
}
