package ee.yoursit.core.command.discord;

import dev.inventex.discord.command.slash.CommandContext;
import dev.inventex.discord.command.slash.SlashCommand;
import dev.inventex.discord.command.slash.SlashCommandInfo;
import ee.yoursit.core.YsmpCore;
import ee.yoursit.core.database.entity.impl.User;
import ee.yoursit.core.util.Constants;
import net.dv8tion.jda.api.EmbedBuilder;
import net.dv8tion.jda.api.entities.Guild;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.MessageEmbed;
import net.dv8tion.jda.api.entities.Role;
import net.dv8tion.jda.api.interactions.commands.OptionType;

@SlashCommandInfo(name = "unban", description = "Revoke the ban of a player.")
public class UnbanCommand extends SlashCommand {
    @Override
    protected void initialize() {
        addOption(
            OptionType.USER,
            "target",
            "Target user to be removed",
            true, false
        );
    }

    @Override
    public void execute(CommandContext context) {
        Guild guild = YsmpCore.INSTANCE
            .getClient()
            .getJda()
            .getGuildById(Constants.YSMP_SERVER);
        assert guild != null : "Unable to fetch YSMP guild";

        Role role = guild.getRoleById(Constants.BANNED);
        assert role != null : "Unable to fetch Banned role";

        Member member = guild.getMember(context.getUser("target"));
        assert member != null : "Unable to fetch target member";

        guild
            .removeRoleFromMember(member, role)
            .queue();

        YsmpCore.INSTANCE
            .getUserService()
            .getUser(member.getNickname())
            .then(user -> unbanUser(context, user))
            .except(Throwable::printStackTrace);
    }

    private void unbanUser(CommandContext context, User user) {
        user.getBanned().set(false);

        MessageEmbed embed = new EmbedBuilder()
            .setTitle("User Unbanned")
            .setColor(Constants.TRANSPARENT)
            .addField("Target", user.getUsername().get(), true)
            .addField("Moderator", context.getInteraction().getUser().getName(), true)
            .build();

        context
            .getInteraction()
            .replyEmbeds(embed)
            .queue();
    }
}
