package ee.yoursit.core.data;

import dev.inventex.octa.concurrent.future.Future;
import dev.inventex.octa.concurrent.future.FutureTimeoutException;
import ee.yoursit.core.YsmpCore;
import ee.yoursit.core.config.Messages;
import ee.yoursit.core.database.entity.impl.Group;
import ee.yoursit.core.database.entity.impl.User;
import ee.yoursit.core.database.entity.impl.Verification;
import lombok.Getter;
import lombok.Setter;
import org.bukkit.Bukkit;
import org.bukkit.Location;
import org.bukkit.entity.Player;
import org.jetbrains.annotations.Nullable;

import java.lang.ref.Reference;
import java.lang.ref.WeakReference;
import java.util.*;
import java.util.concurrent.TimeUnit;

@Getter
@Setter
public class PlayerData {
    public static final int TELEPORT_REQUEST_LIFETIME = 1000 * 30;

    private final Map<UUID, Long> teleportRequests = new HashMap<>();

    private final Reference<Player> player;

    private boolean online;
    private long joinTime;

    @Nullable
    private UUID lastMessenger;

    private Future<User> user;

    @Setter
    private boolean verifying;
    private Verification verification;

    public PlayerData(Player player) {
        this.player = new WeakReference<>(player);

        Bukkit.getScheduler().scheduleSyncRepeatingTask(YsmpCore.INSTANCE.getPlugin(),
            this::cleanUpExpiredRequests, 20L, 20L);

        user = YsmpCore.INSTANCE
            .getUserService()
            .getUser(player.getUniqueId())
            .timeout(3, TimeUnit.SECONDS)
            .except(this::handleUserNotFound);
    }

    private void handleUserNotFound(Throwable err) {
        Player player = this.player.get();
        if (player == null)
            return;

        if (err instanceof FutureTimeoutException) {
            kick(Messages.UNABLE_TO_FETCH);
            return;
        }

        try {
            Verification verification = YsmpCore.INSTANCE
                .getVerificationService()
                .getVerification(player)
                .getOrDefault(null);

            if (verification != null) {
                handleVerification(player, verification);
                return;
            }
        } catch (Exception ignored) {
        }

        kick(Messages.NOT_WHITELISTED);
    }

    private void handleVerification(Player player, Verification verification) {
        try {
            teleport(YsmpCore.INSTANCE.getVerifyLoc());
            player.sendMessage(Messages.INVITE_USE_VERIFY);

            verifying = true;
            this.verification = verification;

            long timeout = YsmpCore.INSTANCE
                .getConfig()
                .getLong("verify-timeout");

            Bukkit
                .getScheduler()
                .scheduleSyncDelayedTask(YsmpCore.INSTANCE.getPlugin(), () -> {
                    if (verifying)
                        kick(Messages.INVITE_VERIFY_TIMEOUT);
                }, timeout * 20);

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void updateLastOnline() {
        user.then(u -> u
            .getLastOnline()
            .set(System.currentTimeMillis()));
    }

    public Future<Group> getGroup() {
        return getUser().tryTransform(user ->
            YsmpCore.INSTANCE
                .getGroupService()
                .getGroup(user.getGroup().get())
                .get());
    }

    public void addTeleportRequest(Player requester) {
        teleportRequests.put(requester.getUniqueId(), System.currentTimeMillis());
    }

    public void removeTeleportRequest(Player requester) {
        teleportRequests.remove(requester.getUniqueId());
    }

    @Nullable
    public Long getTeleportRequest(Player requester) {
        return teleportRequests.get(requester.getUniqueId());
    }

    private void cleanUpExpiredRequests() {
        long now = System.currentTimeMillis();
        teleportRequests
            .entrySet()
            .removeIf(entry -> now - entry.getValue() > TELEPORT_REQUEST_LIFETIME);
    }

    public Player getLastMessenger() {
        if (lastMessenger == null)
            return null;
        return Bukkit.getPlayer(lastMessenger);
    }

    public void kick(String reason) {
        sync(() -> {
            Player player = this.player.get();
            if (player != null)
                player.kickPlayer(reason);
        });
    }

    public void teleport(Location location) {
        sync(() -> {
            Player player = this.player.get();
            if (player != null)
                player.teleport(location);
        });
    }

    private void sync(Runnable task) {
        Bukkit
            .getScheduler()
            .runTask(YsmpCore.INSTANCE.getPlugin(), task);
    }
}
