package ee.yoursit.core.database.entity.api;

import com.mongodb.client.model.Updates;
import dev.inventex.octa.concurrent.future.Future;
import lombok.Getter;
import lombok.RequiredArgsConstructor;
import org.bson.conversions.Bson;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@RequiredArgsConstructor
public class EntityProperty<T> {
    private static final Map<Class<?>, Object> FALLBACK = new HashMap<>();
    static {
        FALLBACK.put(Byte.class, (byte) 0);
        FALLBACK.put(Short.class, (short) 0);
        FALLBACK.put(Integer.class, 0);
        FALLBACK.put(Float.class, 0F);
        FALLBACK.put(Double.class, 0D);
        FALLBACK.put(Long.class, 0L);
        FALLBACK.put(Boolean.class, false);
        FALLBACK.put(Character.class, (char) 0);
        FALLBACK.put(Number.class, 0);
        FALLBACK.put(List.class, new ArrayList<>());
        FALLBACK.put(Map.class, new HashMap<>());
    }

    private final Entity entity;
    private final String name;

    @Getter
    private final Class<T> type;

    public T value;
    private boolean init = false;

    public EntityProperty(Entity entity, String name, Class<T> type, T value) {
        this(entity, name, type);
        init(value);
    }

    @SuppressWarnings("unchecked")
    public final T get() {
        if (!init && FALLBACK.containsKey(type))
            return (T) FALLBACK.get(type);
        return value;
    }

    @MutateEntity
    public final Future<Void> set(T value) {
        this.value = value;
        init = true;
        return mutate();
    }

    public final void init(T value) {
        this.value = value;
        init = true;
    }

    @MutateEntity
    public final Future<Void> mutate() {
        return Future.completeAsync(() -> {
            try {
                Object serialize = EntitySerializer.serialize(type, value);
                entity
                    .collection()
                    .updateOne(entity.query(), Updates.set(name, serialize));
            } catch (Exception e) {
                e.printStackTrace();
            }
        });
    }
}
