package ee.yoursit.core.module;

import ee.yoursit.core.command.bukkit.api.Command;
import ee.yoursit.core.command.bukkit.impl.admin.GroupCommand;
import ee.yoursit.core.command.bukkit.impl.home.DelHomeCommand;
import ee.yoursit.core.command.bukkit.impl.home.HomeCommand;
import ee.yoursit.core.command.bukkit.impl.home.SetHomeCommand;
import ee.yoursit.core.command.bukkit.impl.invite.VerifyCommand;
import ee.yoursit.core.command.bukkit.impl.message.MessageCommand;
import ee.yoursit.core.command.bukkit.impl.message.ReplyCommand;
import ee.yoursit.core.command.bukkit.impl.teleport.TeleportAccept;
import ee.yoursit.core.command.bukkit.impl.teleport.TeleportDeny;
import ee.yoursit.core.command.bukkit.impl.teleport.TeleportRequest;

import java.lang.reflect.Constructor;
import java.util.ArrayList;
import java.util.List;

public class BukkitCommandRegistry implements CommandRegistry {
    private final Class<?>[] classes = {
        TeleportRequest.class,
        TeleportAccept.class,
        TeleportDeny.class,

        MessageCommand.class,
        ReplyCommand.class,

        SetHomeCommand.class,
        DelHomeCommand.class,
        HomeCommand.class,

        VerifyCommand.class,

        GroupCommand.class
    };

    private final Constructor<?>[] constructors = new Constructor[classes.length];

    private final List<Command> commands = new ArrayList<>(classes.length);

    public BukkitCommandRegistry() {
        for (int i = 0; i < classes.length; i++) {
            Class<?> command = classes[i];
            try {
                constructors[i] = command.getDeclaredConstructor();
            } catch (NoSuchMethodException e) {
                System.err.println("Unable to load command " + command.getSimpleName() + ":");
                e.printStackTrace();
            }
        }
    }

    @Override
    public void load() {
        for (Constructor<?> constructor : constructors) {
            try {
                Command command = (Command) constructor.newInstance();
                commands.add(command);
                command.register();
            } catch (Exception e) {
                System.err.println("Unable to instantiate check " + constructor.getDeclaringClass().getSimpleName());
                e.printStackTrace();
            }
        }
    }

    @Override
    public void unload() {
        for (Command command : commands)
            command.unregister();
    }
}
