package ee.yoursit.core.module;

import dev.inventex.octa.concurrent.future.Future;
import ee.yoursit.core.YsmpCore;
import ee.yoursit.core.config.Messages;
import ee.yoursit.core.data.PlayerData;
import ee.yoursit.core.database.entity.impl.Group;
import ee.yoursit.core.database.entity.impl.User;
import ee.yoursit.core.util.Message;
import org.bukkit.Bukkit;
import org.bukkit.entity.Player;
import org.bukkit.event.Cancellable;
import org.bukkit.event.EventHandler;
import org.bukkit.event.Listener;
import org.bukkit.event.player.*;
import org.bukkit.event.server.ServerListPingEvent;
import org.bukkit.util.Vector;

import java.util.Collection;
import java.util.HashSet;
import java.util.Set;
import java.util.UUID;

public class BukkitListener implements Listener {
    private final Set<UUID> sleeping = new HashSet<>();

    @EventHandler
    public void onLogin(PlayerLoginEvent event) {
        YsmpCore.INSTANCE
            .getDataManager()
            .add(event.getPlayer());
    }

    @EventHandler
    public void onJoin(PlayerJoinEvent event) {
        Player player = event.getPlayer();
        PlayerData data = YsmpCore.INSTANCE
            .getDataManager()
            .get(player);

        data.setOnline(true);
        data.setJoinTime(System.currentTimeMillis());
        data.updateLastOnline();

        String message = Messages
            .PLAYER_JOIN
            .replace("{player}", player.getName());

        event.setJoinMessage(message);
    }

    @EventHandler
    public void onQuit(PlayerQuitEvent event) {
        Player player = event.getPlayer();

        YsmpCore.INSTANCE
            .getDataManager()
            .get(player)
            .updateLastOnline();

        YsmpCore.INSTANCE
            .getDataManager()
            .remove(player);

        String message = Messages
            .PLAYER_LEAVE
            .replace("{player}", player.getName());

        event.setQuitMessage(message);
    }

    @EventHandler
    public void onBedEnter(PlayerBedEnterEvent event) {
        Collection<? extends Player> onlinePlayers = Bukkit.getOnlinePlayers();
        long sleeping = onlinePlayers
            .stream()
            .filter(Player::isSleeping)
            .count();
        long total = onlinePlayers.size();

        float ratio = (float) sleeping / (float) total;
        if (ratio >= 2F / 3F) {
            event
                .getPlayer()
                .getWorld()
                .setTime(0);
            onlinePlayers.forEach(player -> player.sendMessage(Messages.AUTO_SLEEP));
        }
    }

    @EventHandler
    public void onPing(ServerListPingEvent event) {
        String motd = Message.translate(
    """
            &b&lYourSitee SMP &f&l❘ 1.20.1
            &f ▶ Visit yoursit.ee/smp for more info
            """
        );
        event.setMotd(motd);
    }

    @EventHandler
    public void onMove(PlayerMoveEvent event) {
        Vector from = event.getFrom().toVector();
        Vector to = event.getTo().toVector();

        double distance = from.distance(to);
        if (distance < .01)
            return;

        cancelVerifying(event.getPlayer(), event);
    }

    @EventHandler
    public void onCommand(PlayerCommandPreprocessEvent event) {
        String command = event
            .getMessage()
            .split(" ")[0]
            .substring(1);

        if (command.equals("verify"))
            return;

        cancelVerifying(event.getPlayer(), event);
    }

    @EventHandler
    public void onChat(AsyncPlayerChatEvent event) {
        Player player = event.getPlayer();
        cancelVerifying(player, event);

        PlayerData data = YsmpCore.INSTANCE
            .getDataManager()
            .get(player);

        Group group = null;
        try {
            User user = data
                .getUser()
                .get();

            group = YsmpCore.INSTANCE
                .getGroupService()
                .getGroup(user.getGroup().get())
                .get();
        } catch (Exception e) {
            e.printStackTrace();
        }

        String prefix = null, suffix = null;
        if (group != null) {
            prefix = group.getPrefix().get();
            suffix = group.getSuffix().get();
        }
        if (prefix == null) prefix = "";
        if (suffix == null) suffix = "";

        String format = "&r" + prefix + player.getName() + "&r" + suffix + "&7: &r" + event.getMessage();
        event.setFormat(Message.translate(format));
    }

    private void cancelVerifying(Player player, Cancellable cancellable) {
        PlayerData data = YsmpCore.INSTANCE
            .getDataManager()
            .get(player);

        if (data.isVerifying())
            cancellable.setCancelled(true);
    }
}
